"""
    
    Reference widget where referred UIDs can be typed in manually
    
    Copyright 2005 Mikko Ohtamaa

"""


from AccessControl import ClassSecurityInfo
from Products.Archetypes.public import ReferenceWidget
from Products.Archetypes.public import StringWidget
from Products.Archetypes.Registry import registerWidget

def parseStringList(input, widget):     
    """ Turns comma separated string to Python list object 
    
    A helper fuction for manipulating user HTML form input.
    
    Parameters:
        - input: a text string to be parsed
        - widget: widget whose properties (change_case) are used for parsing
    """
    list = []
    
    if input == None or input == "":
        return list
                    
    #print "widget:" + str(widget)
        
    # get case changing property from the widget
    case = widget.change_case
    
    # there might be a lonely space, get rid of it
    input = input.strip()
    
    splitted = input.split(",")  
              
    for s in splitted:
        if case == "upper":
            s = s.upper()
        elif case == "lower":
            s = s.lower()        
        list.append(s.strip())
    return list
    

class TypedReferenceWidget(ReferenceWidget):
    """ Reference widget where referred UIDs can be typed in manually   
    
    TypedReferenceWidget is a modified ReferenceWidget.            
    Instead getting references selected using checkboxes,
    references are typed in manually using a comma separated list.
    
    This is useful for cases where there are tons of possible references.
    
    Field using TypedReferenceWidget should be accomppanied with UIDListValidator validator.
    Othetwise an user can type in a string which is not a proper UID and an exception is 
    thrown. See UIDListValidator comments. 
    
    A new property, change_case was introduced. Possible values are None, 'lower', 'upper'.
    When editing reference list, the text box value is uppercased/lowercased according to this
    field. This is useful, if unique ids are always in lowercase or uppercase.
    """


    _properties = ReferenceWidget._properties.copy()

    _properties.update({
        # Skin file used for this widget
        'macro' : "typedreference",
        # Introduce change_case property, can be "upper", "lower" or None
        'change_case' : None,
        })
        
    security = ClassSecurityInfo()
    
    security.declarePublic('getCommaSeparatedEditList')                        
    def getCommaSeparatedEditList(self, field):
        """ Return reference list as text for the edit box """
        refs = self.getReferenceImpl(field.relationship)
        value = ""
        for n in range(0, len(refs)):
            ref = refs[n]
            value += ref.getTargetObject().getId()
            if n != len(refs)-1:
                value += ", "
        
        return value
            
            
    security.declarePublic('process_form')
    def process_form(self, instance, field, form, empty_marker=None,
                     emptyReturnsMarker=False):
        """ Read in a text string and spit out a UID list """
        
        #print "typedrefwidget"
        #print "inst:" + str(instance)
        #print "field:" + str(field)
        #print "form:" + str(form)
               
        value = form.get(field.getName(), empty_marker)
                                
        if value is empty_marker:
            return empty_marker
        if emptyReturnsMarker and value == '':
            return empty_marker
                                       
        # text field value is the first item in form.get return list
        return parseStringList(value[0], self), {}               

registerWidget(TypedReferenceWidget,
               title='TypedReferenceWidget',
               description=('Reference widget where referred UIDs can be typed in manually'),
               used_for=None)