"""

    Typed in UID list validator

    Copyright 2005 Mikko Ohtamaa
    
"""
from Products.Archetypes.interfaces.referenceable import IReferenceable
from Products.CMFCore.utils import getToolByName
from Products.validation.interfaces import ivalidator
from Products.TypedReferenceWidget.TypedReferenceWidget import parseStringList

def lookupObject(catalog, uid):
    """ Look-up an object using Archetypes UID
    
    A helper function.
    
    Taken from:
    http://64.233.183.104/search?q=cache:ZVvM9StpBT8J:developer.etria.com/projects/quills/svn/trunk/BloggerAPI.py%3Frevision:int%3D1215+%22uid_catalog%22+uid&hl=en&lr=lang_en|lang_fi&client=opera    
    
    Deprecated example:
    Taken from http://plone.org/documentation/archetypes/ArchetypesDeveloperGuide/index_html#about-uids
    
    There might be a smarter way nowadays.
    
    Parameters:
        - catalog: portal.portal_catalog object
    """
    if not uid:
        return None
    object = None
    result  = catalog(UID=uid)
    if result:
        #This is an awful workaround for the UID under containment
        #problem. NonRefs will aq there parents UID which is so
        #awful I am having trouble putting it into words.
        for object in result:
            o = object.getObject()
            if o is not None:
                if IReferenceable.isImplementedBy(o):
                    return o
    return None
 
class UIDListValidator:
    """ Validates that in a comma separeted list all entries are valid Archetypes unique object identifies
    
    Preferably associate UIDListValidator with a field by giving a direct instance:
    
    Example::
    
      from Products.TypedReferenceWidget.UIDListValidator import UIDListValidator
    
      ReferenceField("relatedTo",
            relationship="reference",
            validators=UIDListValidator(),
            widget=TypedReferenceWidget(description="Related to",       
                label="Related issues",
            ),
        ),
    
    
    """
    __implements__ = (ivalidator,)
 
    def __init__(self, name):
        self.name = name
        self.title = self.name
        self.description = "Validates that in a comma separeted list all entries are valid Archetypes unique object identifies"
                
    def __call__(self, value, *args, **kwargs):
        """ Validate that all typed in ids can be found from Archetypes UID catalog """
        #print "UIDListValidator"
        #print str(args)
        #print str(kwargs)
                
        request =  kwargs["REQUEST"]            
        instance = kwargs["instance"]
        field = kwargs["field"]
        form = request.form                                
        # text field value is the first item in form.get return list
        value = form.get(field.getName())[0]
        
        #print "val:" + str(value)
        
        # Get tool managing all Archetypes UIDs
        catalog = getToolByName(instance, 'uid_catalog')
        
        # Convert comma separated string to string list
        list = parseStringList(value, field.widget)
        
        for uid in list:
            # Check that every id is found in the catalog
            object = lookupObject(catalog, uid)
            if object == None:
               return "The portal doesn't have an object whose UID is %s. Type in a comma separated list of existing object UIDs." % uid
               
               
        return None
